use std::fmt::{Debug, Display, Formatter};

#[derive(Debug)]
pub enum ErrorKind {
    Database,
    Config,
    Template,
    NotFound,
    Exists,
    Tran,
}

#[derive(Debug)]
pub struct Error {
    pub kind: ErrorKind,
    pub message: String,
    pub cause: Option<Box<dyn std::error::Error>>,
}

impl axum::response::IntoResponse for Error {
    fn into_response(self) -> axum::response::Response {
        self.message.into_response()
    }
}

impl Error {
    pub fn new(kind: ErrorKind, message: String, cause: Option<Box<dyn std::error::Error>>) -> Self {
        Self { kind, message, cause }
    }

    pub fn with_cause(cause: Box<dyn std::error::Error>, kind: ErrorKind) -> Self {
        Self::new(kind, cause.to_string(), Some(cause))
    }

    pub fn not_found(msg: &str) -> Self {
        Self::new(ErrorKind::NotFound, msg.to_string(), None)
    }

    pub fn exists(msg: &str) -> Self {
        Self::new(ErrorKind::Exists, msg.to_string(), None)
    }

    pub fn tran(msg: &str) -> Self {
        Self::new(ErrorKind::Tran, msg.to_string(), None)
    }
}

impl Display for Error {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        write!(f, "{:?}", self)
    }
}

impl std::error::Error for Error {}

impl From<sqlx::Error> for Error {
    fn from(err: sqlx::Error) -> Self {
        Self::with_cause(Box::new(err), ErrorKind::Database)
    }
}


impl From<config::ConfigError> for Error {
    fn from(err: config::ConfigError) -> Self {
        Self::with_cause(Box::new(err), ErrorKind::Config)
    }
}

impl From<askama::Error> for Error {
    fn from(err: askama::Error) -> Self {
        Self::with_cause(Box::new(err), ErrorKind::Template)
    }
}

